<?php
/*
    Implementation of basic Gram CRM API in PHP using CURL

    API: https://www.gramcrm.sk/api/
    web: https://www.gramcrm.sk

    2022-01-03  Initial release
*/

class crmApi {
    const API_URL = 'https://api.gramcrm.com/%s/api/v1/';
    const API_RESULT = 'Result';
    const HTTP_GET = 'POST';
    const HTTP_PUT = 'PUT';
    const HTTP_DELETE = 'DELETE';

    private $organization;
    private $token;

    public function __construct($organization, $token) {
        $this->organization = $organization;
        $this->token = $token;                
    }

    public function callApi($url, $method, $data = array()) {
        $curl = curl_init();	
        curl_setopt_array($curl,
            array (
                CURLOPT_SSL_VERIFYPEER => FALSE,
                CURLOPT_SSL_VERIFYHOST => FALSE,
                CURLOPT_URL => sprintf(self::API_URL. $url, $this->organization),
                CURLOPT_RETURNTRANSFER => TRUE,
                CURLOPT_ENCODING => "",
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 30,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => $method,  
                CURLOPT_POSTFIELDS => json_encode($data),
                CURLOPT_HTTPHEADER => array (
                    "cache-control: no-cache",
                    "content-type: application/json",
                    "Token: ". $this->token .""
                ),
            )
        );        
        $response = curl_exec($curl);
        $error = curl_error($curl);        
        curl_close($curl);            
        return json_decode($response, TRUE);
    }
    
    public function getOptionList($codelistName) {
        // TODO:         
        return '';
    }
    
    public function createEntity($name, $data) {
        return $this->callApi($name, self::HTTP_PUT, $data);
    }

    public function getEntity($name, $id = '', $data = array()) {
        $url = $name;
        if ($id !== '') $url =  $url .'/'. $id;
        return $this->callApi($url, self::HTTP_GET, $data);
    }

    public function uploadFile($name, $body, $type = 'text/plain') {        
        $curl = curl_init();	
        $fullUrl = sprintf(self::API_URL. 'File?Name=', $this->organization). rawurlencode($name);
        curl_setopt_array($curl,
            array (
                CURLOPT_SSL_VERIFYPEER => FALSE,
                CURLOPT_SSL_VERIFYHOST => FALSE,
                CURLOPT_URL => $fullUrl,
                CURLOPT_RETURNTRANSFER => TRUE,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 30,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'POST',
                CURLOPT_POSTFIELDS => $body,
                CURLOPT_HTTPHEADER => array (
                    'cache-control: no-cache',
                    'content-type: '. $type,
                    'Token: '. $this->token
                ),
            )
        );        
        $response = curl_exec($curl);
        curl_close($curl);            
        $res = json_decode($response, TRUE);
        return $res;
    }        
}